@@ -23,8 +23,8 @@
 # Copyright (c) 2018 Gary Tyler McLeod <mail@garytyler.com>
 # Copyright (c) 2018 Konstantin <Github@pheanex.de>
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2019, 2021 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019, 2021 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2019 Janne Rönkkö <jannero@users.noreply.github.com>
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 # Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
@@ -40,15 +40,18 @@ import pickle
 import sys
 from datetime import datetime
 
-import platformdirs
-
 from pylint.config.configuration_mixin import ConfigurationMixIn
-from pylint.config.find_default_config_files import find_default_config_files
+from pylint.config.find_default_config_files import (
+    find_default_config_files,
+    find_pylintrc,
+)
 from pylint.config.man_help_formatter import _ManHelpFormatter
 from pylint.config.option import Option
 from pylint.config.option_manager_mixin import OptionsManagerMixIn
 from pylint.config.option_parser import OptionParser
 from pylint.config.options_provider_mixin import OptionsProviderMixIn, UnsupportedAction
+from pylint.constants import DEFAULT_PYLINT_HOME, OLD_DEFAULT_PYLINT_HOME
+from pylint.utils import LinterStats
 
 __all__ = [
     "ConfigurationMixIn",
@@ -67,9 +70,9 @@ if "PYLINTHOME" in os.environ:
     if USER_HOME == "~":
         USER_HOME = os.path.dirname(PYLINT_HOME)
 elif USER_HOME == "~":
-    PYLINT_HOME = ".pylint.d"
+    PYLINT_HOME = OLD_DEFAULT_PYLINT_HOME
 else:
-    PYLINT_HOME = platformdirs.user_cache_dir("pylint")
+    PYLINT_HOME = DEFAULT_PYLINT_HOME
     # The spam prevention is due to pylint being used in parallel by
     # pre-commit, and the message being spammy in this context
     # Also if you work with old version of pylint that recreate the
@@ -79,7 +82,7 @@ else:
         PYLINT_HOME,
         datetime.now().strftime(prefix_spam_prevention + "_%Y-%m-%d.temp"),
     )
-    old_home = os.path.join(USER_HOME, ".pylint.d")
+    old_home = os.path.join(USER_HOME, OLD_DEFAULT_PYLINT_HOME)
     if os.path.exists(old_home) and not os.path.exists(spam_prevention_file):
         print(
             f"PYLINTHOME is now '{PYLINT_HOME}' but obsolescent '{old_home}' is found; "
@@ -117,17 +120,20 @@ def load_results(base):
     data_file = _get_pdata_path(base, 1)
     try:
         with open(data_file, "rb") as stream:
-            return pickle.load(stream)
+            data = pickle.load(stream)
+            if not isinstance(data, LinterStats):
+                raise TypeError
+            return data
     except Exception:  # pylint: disable=broad-except
-        return {}
+        return None
 
 
 def save_results(results, base):
     if not os.path.exists(PYLINT_HOME):
         try:
-            os.mkdir(PYLINT_HOME)
+            os.makedirs(PYLINT_HOME)
         except OSError:
-            print("Unable to create directory %s" % PYLINT_HOME, file=sys.stderr)
+            print(f"Unable to create directory {PYLINT_HOME}", file=sys.stderr)
     data_file = _get_pdata_path(base, 1)
     try:
         with open(data_file, "wb") as stream:
@@ -136,24 +142,4 @@ def save_results(results, base):
         print(f"Unable to create file {data_file}: {ex}", file=sys.stderr)
 
 
-def find_pylintrc():
-    """search the pylint rc file and return its path if it find it, else None"""
-    for config_file in find_default_config_files():
-        if config_file.endswith("pylintrc"):
-            return config_file
-
-    return None
-
-
 PYLINTRC = find_pylintrc()
-
-ENV_HELP = """
-The following environment variables are used:
-    * PYLINTHOME
-    Path to the directory where persistent data for the run will be stored. If
-not found, it defaults to ~/.pylint.d/ or .pylint.d (in the current working
-directory).
-    * PYLINTRC
-    Path to the configuration file. See the documentation for the method used
-to search for configuration file.
-"""
