@@ -5,22 +5,23 @@ import os
 import sys
 import warnings
 
-from pylint import __pkginfo__, config, extensions, interfaces
-from pylint.constants import full_version
+from pylint import __pkginfo__, extensions, interfaces
+from pylint.constants import DEFAULT_PYLINT_HOME, OLD_DEFAULT_PYLINT_HOME, full_version
 from pylint.lint.pylinter import PyLinter
 from pylint.lint.utils import ArgumentPreprocessingError, preprocess_options
-from pylint.utils import utils
+from pylint.utils import print_full_documentation, utils
 
 try:
     import multiprocessing
+    from multiprocessing import synchronize  # noqa pylint: disable=unused-import
 except ImportError:
-    multiprocessing = None  # type: ignore
+    multiprocessing = None  # type: ignore[assignment]
 
 
 def _cpu_count() -> int:
     """Use sched_affinity if available for virtualized or containerized environments."""
     sched_getaffinity = getattr(os, "sched_getaffinity", None)
-    # pylint: disable=not-callable,using-constant-test
+    # pylint: disable=not-callable,using-constant-test,useless-suppression
     if sched_getaffinity:
         return len(sched_getaffinity(0))
     if multiprocessing:
@@ -40,7 +41,7 @@ def cb_list_extensions(option, optname, value, parser):
 
 def cb_list_confidence_levels(option, optname, value, parser):
     for level in interfaces.CONFIDENCE_LEVELS:
-        print("%-18s: %s" % level)
+        print(f"%-18s: {level}")
     sys.exit(0)
 
 
@@ -92,6 +93,7 @@ group are mutually exclusive.",
                     "init-hook": (cb_init_hook, True),
                     "rcfile": (self.cb_set_rcfile, True),
                     "load-plugins": (self.cb_add_plugins, True),
+                    "enable-all-extensions": (self.cb_enable_all_extensions, False),
                     "verbose": (self.cb_verbose_mode, False),
                     "output": (self.cb_set_output, True),
                 },
@@ -247,16 +249,6 @@ group are mutually exclusive.",
                         "displayed, and no reports are done by default.",
                     },
                 ),
-                (
-                    "py3k",
-                    {
-                        "action": "callback",
-                        "callback": self.cb_python3_porting_mode,
-                        "help": "In Python 3 porting mode, all checkers will be "
-                        "disabled and only messages emitted by the porting "
-                        "checker will be displayed.",
-                    },
-                ),
                 (
                     "verbose",
                     {
@@ -267,6 +259,15 @@ group are mutually exclusive.",
                         "will be displayed.",
                     },
                 ),
+                (
+                    "enable-all-extensions",
+                    {
+                        "action": "callback",
+                        "callback": self.cb_enable_all_extensions,
+                        "help": "Load and enable all available extensions. "
+                        "Use --list-extensions to see a list all available extensions.",
+                    },
+                ),
             ),
             option_groups=self.option_groups,
             pylintrc=self._rcfile,
@@ -279,7 +280,20 @@ group are mutually exclusive.",
         # load command line plugins
         linter.load_plugin_modules(self._plugins)
         # add some help section
-        linter.add_help_section("Environment variables", config.ENV_HELP, level=1)
+        linter.add_help_section(
+            "Environment variables",
+            f"""
+The following environment variables are used:
+    * PYLINTHOME
+    Path to the directory where persistent data for the run will be stored. If
+not found, it defaults to '{DEFAULT_PYLINT_HOME}' or '{OLD_DEFAULT_PYLINT_HOME}'
+(in the current working directory).
+    * PYLINTRC
+    Path to the configuration file. See the documentation for the method used
+to search for configuration file.
+""",
+            level=1,
+        )
         linter.add_help_section(
             "Output",
             "Using the default text output, the message format is :                          \n"
@@ -350,8 +364,7 @@ group are mutually exclusive.",
 
         if linter.config.jobs < 0:
             print(
-                "Jobs number (%d) should be greater than or equal to 0"
-                % linter.config.jobs,
+                f"Jobs number ({linter.config.jobs}) should be greater than or equal to 0",
                 file=sys.stderr,
             )
             sys.exit(32)
@@ -375,7 +388,7 @@ group are mutually exclusive.",
         if self._output:
             try:
                 with open(self._output, "w", encoding="utf-8") as output:
-                    linter.reporter.set_output(output)
+                    linter.reporter.out = output
                     linter.check(args)
                     score_value = linter.generate_reports()
             except OSError as ex:
@@ -399,8 +412,13 @@ group are mutually exclusive.",
                 # We need to make sure we return a failing exit code in this case.
                 # So we use self.linter.msg_status if that is non-zero, otherwise we just return 1.
                 sys.exit(self.linter.msg_status or 1)
-            elif score_value is not None and score_value >= linter.config.fail_under:
-                sys.exit(0)
+            elif score_value is not None:
+                if score_value >= linter.config.fail_under:
+                    sys.exit(0)
+                else:
+                    # We need to make sure we return a failing exit code in this case.
+                    # So we use self.linter.msg_status if that is non-zero, otherwise we just return 1.
+                    sys.exit(self.linter.msg_status or 1)
             else:
                 sys.exit(self.linter.msg_status)
 
@@ -447,7 +465,7 @@ group are mutually exclusive.",
 
     def cb_full_documentation(self, option, optname, value, parser):
         """optik callback for printing full documentation"""
-        self.linter.print_full_documentation()
+        print_full_documentation(self.linter)
         sys.exit(0)
 
     def cb_list_messages(self, option, optname, value, parser):
@@ -470,9 +488,19 @@ group are mutually exclusive.",
             print(check)
         sys.exit(0)
 
-    def cb_python3_porting_mode(self, *args, **kwargs):
-        """Activate only the python3 porting checker."""
-        self.linter.python3_porting_mode()
-
     def cb_verbose_mode(self, *args, **kwargs):
         self.verbose = True
+
+    def cb_enable_all_extensions(self, option_name: str, value: None) -> None:
+        """Callback to load and enable all available extensions"""
+        for filename in os.listdir(os.path.dirname(extensions.__file__)):
+            # pylint: disable=fixme
+            # TODO: Remove the check for deprecated check_docs after the extension has been removed
+            if (
+                filename.endswith(".py")
+                and not filename.startswith("_")
+                and not filename.startswith("check_docs")
+            ):
+                extension_name = f"pylint.extensions.{filename[:-3]}"
+                if extension_name not in self._plugins:
+                    self._plugins.append(extension_name)
